package com.persagy.init;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.ResourceUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.Feature;
import com.persagy.cache.CurrentDataCache;
import com.persagy.constant.CommonConst;
import com.persagy.entity.PonitData;
import com.persagy.entity.ReturnAlarmDefine;

import cn.hutool.core.util.BooleanUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 项目启动后，加载控返不一致报警所需数据
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年10月14日 下午5:09:18
 */
@Slf4j
@Component
public class InitReturnAlarmConfig implements CommandLineRunner {
	
	public static volatile boolean alarmEnable = false;
	
	/** 信息点到报警定义 */
	public static Map<String, ReturnAlarmDefine> infoCode2Alarm = new HashMap<String, ReturnAlarmDefine>();
	/** 信息点到报警定义 */
	public static Map<String, ReturnAlarmDefine> returnCode2Alarm = new HashMap<String, ReturnAlarmDefine>();
	/** 信息点到报警定义 */
	public static Map<String, ReturnAlarmDefine> alarmCode2Alarm = new HashMap<String, ReturnAlarmDefine>();
	
	/** 表号-功能号, 信息点 */
	public static Map<String, String> meterFunid2Code = new HashMap<String, String>();
	
	/** 报警信息点, 1-报警，0-正常，-1-不作任何处理 */
	public static Map<String, Integer> alarmCode2IsAlarm = new ConcurrentHashMap<String, Integer>();
	
	@Override
	public void run(String... args) throws Exception {
		if (!CommonConst.collectEnable) {
			return;
		}
		// 获取控返不一致报警配置文件
		File file = this.getFile();
		if (file == null || !file.exists()) {
			return;
		}
		// 解析文件
		this.parseAlarmConfig(file);
	}
	
	/**
	 * 文件加载，并放入本地缓存
	 * 
	 * @param alarmFile
	 * @throws IOException 
	 * @date 2021年10月14日 下午5:35:03
	 */
	private void parseAlarmConfig(File alarmFile) throws IOException {
		JSONObject fileJSON = this.readFileInfo(alarmFile);
		alarmEnable = fileJSON.getBoolean("alarm-enable");
		if (!BooleanUtil.isTrue(alarmEnable)) {
			return;
		}
		
		JSONArray alarmDefines = fileJSON.getJSONArray("alarm-define");
		for (int i = 0; i < alarmDefines.size(); i++) {
			JSONObject alarmDefine = alarmDefines.getJSONObject(i);
			
			ReturnAlarmDefine define = new ReturnAlarmDefine();
			define.setGroupCode(CommonConst.groupCode);
			define.setProjectId(CommonConst.projectId);
			define.setBuilding(CommonConst.building);
			
			// 需要判断的信息点
			JSONObject setInfo = alarmDefine.getJSONObject("setInfo");
			PonitData setPonitData = new PonitData();
			setPonitData.setInfoCode(setInfo.getString("code"));
			setPonitData.setMeter(setInfo.getString("meter"));
			setPonitData.setFuncid(Integer.valueOf(setInfo.getString("funcid")));
			define.setSetInfo(setPonitData);
			infoCode2Alarm.put(setInfo.getString("code"), define);
			meterFunid2Code.put(CurrentDataCache.getKey(setInfo.getString("meter"), setInfo.getString("funcid")), setInfo.getString("code"));
			
			// 需要验证的实际信息点
			JSONObject returnInfo = alarmDefine.getJSONObject("returnInfo");
			PonitData returnPonitData = new PonitData();
			returnPonitData.setInfoCode(returnInfo.getString("code"));
			returnPonitData.setMeter(returnInfo.getString("meter"));
			returnPonitData.setFuncid(Integer.valueOf(returnInfo.getString("funcid")));
			define.setReturnInfo(returnPonitData);
			returnCode2Alarm.put(returnInfo.getString("code"), define);
			meterFunid2Code.put(CurrentDataCache.getKey(returnInfo.getString("meter"), returnInfo.getString("funcid")), returnInfo.getString("code"));
			
			// 需要产生报警的信息点
			JSONObject alarmInfo = alarmDefine.getJSONObject("alarmInfo");
			PonitData alarmPonitData = new PonitData();
			alarmPonitData.setInfoCode(alarmInfo.getString("code"));
			alarmPonitData.setMeter(alarmInfo.getString("meter"));
			alarmPonitData.setFuncid(Integer.valueOf(alarmInfo.getString("funcid")));
			define.setAlarmInfo(alarmPonitData);
			alarmCode2Alarm.put(alarmInfo.getString("code"), define);
			alarmCode2IsAlarm.put(alarmInfo.getString("code"), -1);		// 默认不做任何处理
			meterFunid2Code.put(CurrentDataCache.getKey(alarmInfo.getString("meter"), alarmInfo.getString("funcid")), alarmInfo.getString("code"));
			
			define.setAlarmCondition(alarmDefine.getString("alarmCondition"));
		}
	}
	
	/**
	 * 读取文件内容，返回文件信息
	 * 
	 * @return
	 * @throws IOException 
	 * @date 2021年10月14日 下午5:37:29
	 */
	private JSONObject readFileInfo(File file) throws IOException {
		return JSON.parseObject(new FileInputStream(file), StandardCharsets.UTF_8, JSONObject.class,
                // 自动关闭流
                Feature.AutoCloseSource,
                // 允许注释
                Feature.AllowComment,
                // 允许单引号
                Feature.AllowSingleQuotes);
		
	}
	
	
	private File getFile() throws FileNotFoundException {
		File file = null;
		try {
			// 先获取config目录下的文件，获取不到，再获取classpath下的
			file = ResourceUtils.getFile(CommonConst.SERVER_ROOT_PATH + "/config/return-alarm.json");
			if (!file.exists()) {
				log.warn("config目录下不存在控返不一致的报警配置，获取classpath下的默认配置");
				file = ResourceUtils.getFile(ResourceUtils.CLASSPATH_URL_PREFIX + "return-alarm.json");
			}
		} catch (Exception e) {
		}
		return file;
	}
	
}
